unit uParentPage;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ActnList,
  JvTabBar, JvPageList;

type

  { TFormParentPage }

  TFormParentPage = class(TForm)
    actn_CloseTab: TAction;
    actn_ReDockFreeForm: TAction;
    actn_FreeFormFromTab: TAction;
    ActionList1: TActionList;
    Label1: TLabel;
    procedure actn_CloseTabExecute(Sender: TObject);
    procedure actn_FreeFormFromTabExecute(Sender: TObject);
    procedure actn_ReDockFreeFormExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FbIsUniqueWindow: boolean;
    FbShallRefresh: boolean;
    FiFormtype: integer;
    FiID: integer;
     FImageIndex: integer;
     FsTabCaption: String;
     FTabBar:TJvTabBar;
     FPageList:TJvPageList;
     procedure SetbIsUniqueWindow(AValue: boolean);
     procedure SetbShallRefresh(AValue: boolean);
     procedure SetiFormtype(AValue: integer);
     procedure SetiID(AValue: integer);
     procedure SetImageIndex(AValue: integer);
     procedure SetPageList(AValue: TJvPageList);
     procedure SetsTabCaption(AValue: String);
     procedure SetTabBar(AValue: TJvTabBar);
  public
     //die ID bzw der Primärschlüssel der zu der jeweiligen Form gehört
     property piID: integer read FiID write SetiID default -1;
     //die Formtype mit constants aus uconst
     property piFormtype: integer read FiFormtype write SetiFormtype default -1;

     //Soll das Formular vor dem weiterbearbeiten refresht werden? Die Daten wurden möglicherweise von einem anderen Formular in der DB geändert
     //das passiert bei Linkänderungen oder anderen Änderungen die mit einer Person verbiundens ein können.
     property pbShallRefresh:boolean read FbShallRefresh write SetbShallRefresh default false;

     {ein Fenster ist mit piID und piFormType eeindeutig definiert. Fenster können einzigartig, also nur einmal oder mehrmals mit gleicherIF/Typ geöffnet werden
     wenn ein Fenster unique geöffnet werden soll, muss die Prozedur, die es öffnet sie Pagelist durhsuchen um festzustellen ob dieses Fenster schon geöffnet ist.
     ist es schon geöffnet, wird es selektiert, andernfalls neu erzeugt}
     property pbIsUniqueWindow:boolean read FbIsUniqueWindow write SetbIsUniqueWindow default false;

     //macht vielleicht keinen sinn denn eine abfrage auf parent=nil würde das gleiche liefern
//     property pbIsDockedInTab:boolean read FbIsDockedInTab write SetbIsDockedInTab default false;

     property poTabBar:TJvTabBar read FTabBar write SetTabBar;
     property poPageList:TJvPageList read FPageList write SetPageList;
     property piImageIndex:integer read FImageIndex write SetImageIndex default 0;
     property psTabCaption:String read FsTabCaption write SetsTabCaption;
     procedure EngageInTab;
     procedure DisengageTab;
     procedure CloseTab;
  end;

var
  FormParentPage: TFormParentPage;

implementation

{$R *.lfm}

{ein Etikett um Anzeigen und Controls beim Vorgang des Erforschens von
 Funktionen einzuschalten. Im Normalgebrauch ausschalten

 a label to switch on displays and controls when
 exploring functions. Switch off in normal use
 }
{$Define Explore}  //Label for Exploring strategys

{ TFormParentPage }
procedure TFormParentPage.EngageInTab;
var
  aTab:TJvTabBarItem;
  iPageNumber:integer;
begin
  if assigned(Self) then begin
    {offensichtlich wird auch gleichzeitig eine jvStandardpage
     in der Pagelist angelegt und synchronisiert.}
    aTab:=FTabBar.AddTab(FsTabCaption);

    aTab.ImageIndex:=FImageIndex;
    FTabBar.SelectedTab:=aTab;

    iPageNumber:=FTabBar.Tabs.Count-1;
    Self.Parent:=FPageList.Pages[iPageNumber];

    //Pointer des erzeugten Formulars in der Page der Pagelist hinterlegen
    FPageList.Pages[iPageNumber].Tag:=PtrInt(Self);

    Self.BorderStyle:=bsNone;
    Self.Align:=alclient;

    Self.Label1.Caption:=FsTabCaption;
    Self.Show;

    actn_ReDockFreeForm.Enabled:=false;
    actn_FreeFormFromTab.Enabled:=true;
 end
 else
  showmessage('TFormParentPage: Form unassigned'); //For security reasons only. Should not happen

end;

procedure TFormParentPage.DisengageTab;
begin
  Self.parent:=nil;
  Self.Align:=alNone;
  Self.BorderStyle:=bsSizeable;
  Self.Position:=poMainFormCenter;
  FTabBar.CloseTab(FTabBar.SelectedTab);

  actn_ReDockFreeForm.Enabled:=true;
  actn_FreeFormFromTab.Enabled:=false;
end;

procedure TFormParentPage.CloseTab;
begin
  FTabBar.CloseTab(FTabBar.SelectedTab);
end;

procedure TFormParentPage.FormCreate(Sender: TObject);
begin
  FImageIndex:=0;
  FsTabCaption:='unknown Caption';

  actn_ReDockFreeForm.Enabled:=false;
  actn_FreeFormFromTab.Enabled:=true;
  {$ifdef Explore}
     Label1.Visible:=true;
  {$endif}

end;

//********* Actions
procedure TFormParentPage.actn_FreeFormFromTabExecute(Sender: TObject);
begin
  DisengageTab;
end;

procedure TFormParentPage.actn_CloseTabExecute(Sender: TObject);
begin
  CloseTab;
end;

procedure TFormParentPage.actn_ReDockFreeFormExecute(Sender: TObject);
begin
  EngageInTab;
end;

//********* Setter

procedure TFormParentPage.SetTabBar(AValue: TJvTabBar);
begin
  if FTabBar=AValue then Exit;
  FTabBar:=AValue;
  FPageList := TJvPageList(FTabBar.PageList);
end;

procedure TFormParentPage.SetPageList(AValue: TJvPageList);
begin
  if FPageList=AValue then Exit;
  FPageList:=AValue;
end;

procedure TFormParentPage.SetImageIndex(AValue: integer);
begin
  if FImageIndex=AValue then Exit;
  FImageIndex:=AValue;
end;

procedure TFormParentPage.SetbShallRefresh(AValue: boolean);
begin
  if FbShallRefresh=AValue then Exit;
  FbShallRefresh:=AValue;
end;

procedure TFormParentPage.SetbIsUniqueWindow(AValue: boolean);
begin
  if FbIsUniqueWindow=AValue then Exit;
  FbIsUniqueWindow:=AValue;
end;

procedure TFormParentPage.SetiFormtype(AValue: integer);
begin
  if FiFormtype=AValue then Exit;
  FiFormtype:=AValue;
end;

procedure TFormParentPage.SetiID(AValue: integer);
begin
  if FiID=AValue then Exit;
  FiID:=AValue;
end;

procedure TFormParentPage.SetsTabCaption(AValue: String);
begin
  if FsTabCaption=AValue then Exit;
  FsTabCaption:=AValue;
end;

end.

